@@ -49,7 +49,7 @@ fc_PE
 ###################################################
 ### code chunk number 6: Rsubread.Rnw:152-174
 ###################################################
-if(.Platform$OS.type != "windows") {
+if(grepl("linux", R.version$os)) {
   md5.zip <- "ffd5036b36e25e9b61efc412e71820dd"
   URL <- "https://shilab-bioinformatics.github.io/cellCounts-Example/cellCounts-Example.zip"
   temp.file <- tempfile()
